@@ -9,12 +9,8 @@
 #  else
 #    define LIBHUNSPELL_DLL_EXPORTED __declspec(dllimport)
 #  endif
-#elif defined(BUILDING_LIBHUNSPELL)
-#  if BUILDING_LIBHUNSPELL && 1
-#    define LIBHUNSPELL_DLL_EXPORTED __attribute__((__visibility__("default")))
-#  else
-#    define LIBHUNSPELL_DLL_EXPORTED
-#  endif
+#elif defined(BUILDING_LIBHUNSPELL) &&  (BUILDING_LIBHUNSPELL && 1)
+#  define LIBHUNSPELL_DLL_EXPORTED __attribute__((__visibility__("default")))
 #else
 #  define LIBHUNSPELL_DLL_EXPORTED
 #endif
